﻿using System.Windows;   
using System.Windows.Controls;   
using ThinkGeo.MapSuite.Core;   
using ThinkGeo.MapSuite.WpfDesktopEdition;
using System.ComponentModel;
using System.Diagnostics.CodeAnalysis;
using System.Windows.Media.Imaging;
using System;   
  
namespace HelloWorld   
{   
    public partial class MainWindow : Window, INotifyPropertyChanged   
    {
        private int m_Heading = 0;

        public int Heading
        {
            get { return m_Heading; }
            set
            {
                m_Heading = value;
                RaisePropertyChanged("Heading");
            }
        }

        private int m_MinRange = 10;

        public int MinRange
        {
            get { return m_MinRange; }
            set { m_MinRange = value; RaisePropertyChanged("MinRange"); }
        }

        private int m_MaxRange = 50;

        public int MaxRange
        {
            get { return m_MaxRange; }
            set { m_MaxRange = value; RaisePropertyChanged("MaxRange"); }
        }

        private int m_Latitude = 50;

        public int Latitude
        {
            get { return m_Latitude; }
            set { m_Latitude = value; RaisePropertyChanged("Latitude"); }
        }

        private int m_Longitude = 50;

        public int Longitude
        {
            get { return m_Longitude; }
            set { m_Longitude = value; RaisePropertyChanged("Longitude"); }
        }

        public MainWindow()   
        {
            DataContext = this;
            InitializeComponent();
        }   
  
        private void Window_Loaded(object sender, RoutedEventArgs e)   
        {   
            wpfMap1.MapUnit = GeographyUnit.DecimalDegree;   
            wpfMap1.BackgroundOverlay.BackgroundBrush = new GeoSolidBrush(GeoColor.StandardColors.White);

            // Platform
            InMemoryFeatureLayer inMemoryFeatureLayer = new InMemoryFeatureLayer();

            inMemoryFeatureLayer.Open();
            inMemoryFeatureLayer.Columns.Add(new FeatureSourceColumn(defines.ANGLE));
            inMemoryFeatureLayer.Close();

            Feature newFeature1 = new Feature(new PointShape(Longitude, Latitude));
            newFeature1.ColumnValues[defines.ANGLE] = System.Convert.ToString(Heading);
            inMemoryFeatureLayer.InternalFeatures.Add(defines.VEHICLE, newFeature1);

            RotatedImageStyle rotatedImageStyle = new RotatedImageStyle(new GeoImage(@"..\..\Images\Cross.png"), defines.ANGLE);

            inMemoryFeatureLayer.ZoomLevelSet.ZoomLevel01.CustomStyles.Clear();
            inMemoryFeatureLayer.ZoomLevelSet.ZoomLevel01.CustomStyles.Add(rotatedImageStyle);
            inMemoryFeatureLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            LayerOverlay overLayer = new LayerOverlay();
            overLayer.Layers.Add(defines.FEATURE_LAYER, inMemoryFeatureLayer);
            wpfMap1.Overlays.Add(defines.PLATFORM_OVERLAY, overLayer);
   
            // Grid overlay
            GraticuleAdornmentLayer graticuleAdornmentLayer = new GraticuleAdornmentLayer();
            graticuleAdornmentLayer.GraticuleLineStyle.OuterPen.Color = GeoColor.FromArgb(125, GeoColor.StandardColors.Navy);
            wpfMap1.AdornmentOverlay.Layers.Add("graticule", graticuleAdornmentLayer);

            // Scale legend
            ScaleLineAdornmentLayer scaleLineAdornmentLayer = new ScaleLineAdornmentLayer();
            scaleLineAdornmentLayer.Location = AdornmentLocation.LowerLeft;
            wpfMap1.AdornmentOverlay.Layers.Add("ScaleLineAdornmentLayer", scaleLineAdornmentLayer);

            // Marker of centre of platfrom
            SimpleMarkerOverlay markerOverlay = new SimpleMarkerOverlay();
            wpfMap1.Overlays.Add("MarkerOverlay", markerOverlay);


            Marker marker = new Marker(Longitude, Latitude);
            marker.ImageSource = new BitmapImage(new Uri("../../Images/AQUA.png", UriKind.RelativeOrAbsolute));
            marker.Width = 20;
            marker.Height = 34;
            marker.YOffset = -17;

            markerOverlay.Markers.Add(marker);

            // Set view boundaries
            wpfMap1.CurrentExtent = new RectangleShape(0, 100, 100, 0);   
            wpfMap1.Refresh();   
        }
        
        private void Heading_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            if (wpfMap1 == null) return;
            if (!wpfMap1.Overlays.Contains(defines.PLATFORM_OVERLAY)) return;

            LayerOverlay platformOverlay = (LayerOverlay)wpfMap1.Overlays[defines.PLATFORM_OVERLAY];
            InMemoryFeatureLayer inMemoryFeatureLayer = (InMemoryFeatureLayer)platformOverlay.Layers[defines.FEATURE_LAYER];

            Feature feature = inMemoryFeatureLayer.InternalFeatures[defines.VEHICLE];
            feature.ColumnValues[defines.ANGLE] = System.Convert.ToString(Heading);

            wpfMap1.Refresh();
        }

        private void MinRange_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
        }

        private void MaxRange_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {

        }

        private void Position_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            if (wpfMap1 == null) return;
            if (!wpfMap1.Overlays.Contains(defines.PLATFORM_OVERLAY)) return;

            LayerOverlay platformOverlay = (LayerOverlay)wpfMap1.Overlays[defines.PLATFORM_OVERLAY];
            InMemoryFeatureLayer inMemoryFeatureLayer = (InMemoryFeatureLayer)platformOverlay.Layers[defines.FEATURE_LAYER];

            Feature feature = inMemoryFeatureLayer.InternalFeatures[defines.VEHICLE];
            PointShape pointShape1 = feature.GetShape() as PointShape;
            pointShape1.X = Longitude;
            pointShape1.Y = Latitude;
            pointShape1.Id = defines.VEHICLE;

            inMemoryFeatureLayer.Open();
            inMemoryFeatureLayer.EditTools.BeginTransaction();
            inMemoryFeatureLayer.EditTools.Update(pointShape1);
            inMemoryFeatureLayer.EditTools.CommitTransaction();
            inMemoryFeatureLayer.Close();

            wpfMap1.Refresh(wpfMap1.Overlays[defines.PLATFORM_OVERLAY]);
        }

        public event PropertyChangedEventHandler PropertyChanged;
        
        [SuppressMessage("Microsoft.Design", "CA1030:UseEventsWhereAppropriate", Justification = "This cannot be an event")]
        protected virtual void RaisePropertyChanged(string propertyName)
        {
            var handler = PropertyChanged;

            if (handler != null)
            {
                handler(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        private void btnAddAFeature_Click(object sender, RoutedEventArgs e)
        {
            SimpleMarkerOverlay markerOverlay = (SimpleMarkerOverlay)wpfMap1.Overlays["MarkerOverlay"];
            Marker marker = new Marker(Longitude, Latitude);
            marker.ImageSource = new BitmapImage(new Uri("../../Images/AQUA.png", UriKind.RelativeOrAbsolute));
            marker.Width = 20;
            marker.Height = 34;
            marker.YOffset = -17;

            markerOverlay.Markers.Add(marker);
            markerOverlay.Refresh();
        }

    }   
}  
