﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using ThinkGeo.MapSuite.Core;
using System.Windows;

namespace HelloWorld
{
    class RotatedImageStyle : ThinkGeo.MapSuite.Core.Style
    {
        private GeoImage mImage;
        private string mAngleColumnName;

        public RotatedImageStyle()
            : this(new GeoImage(), string.Empty)
        { }

        public RotatedImageStyle(GeoImage geoImage, string angleColumnName)
        {
            mImage = geoImage;
            mAngleColumnName = angleColumnName;
        }

        public GeoImage GeoImage
        {
            get { return mImage; }
            set { GeoImage = value; }
        }

        public string AngleColumnName
        {
            get { return mAngleColumnName; }
            set { AngleColumnName = value; }
        }

        protected override void DrawCore(IEnumerable<Feature> features, GeoCanvas canvas, Collection<SimpleCandidate> labelsInThisLayer, Collection<SimpleCandidate> labelsInAllLayers)
        {
            // An angle of zero is NORTH
            // +ve Angle is clock wise

            float angleData = 0;
            foreach (Feature feature in features)
            {
                try
                {
                    angleData = Convert.ToSingle(feature.ColumnValues[mAngleColumnName]);
                }
                catch (Exception ex)
                {
                    MessageBox.Show(string.Format("The number '{0}' does not parse", feature.ColumnValues[mAngleColumnName]));
                }
                PointShape pointShape = (PointShape)feature.GetShape();
                float heading = convertToNorth(angleData);
                canvas.DrawWorldImageWithoutScaling(mImage, pointShape.X, pointShape.Y, DrawingLevel.LevelFour, 0, 0, heading);
            }
        }

        private float convertToNorth(float p_Angle)
        {
            float angle = 90;
            angle -= p_Angle;
            while (angle < 0)
            {
                angle += 360;
            }
            while (angle > 360)
            {
                angle -= 360;
            }
            return angle;
        }

        protected override Collection<string> GetRequiredColumnNamesCore()
        {
            Collection<string> columns = new Collection<string>();
            if (!columns.Contains(mAngleColumnName))
            {
                columns.Add(mAngleColumnName);
            }
            return columns;
        }
    }
}
